package Apbuild::Utils;

use strict;
use warnings;
use Exporter;
use base qw(Exporter);
use IO::Socket;
use IPC::Open2;
use POSIX;

our @EXPORT = qw(debug empty homeDir searchLib searchStaticLib soname parseDataFile writeDataFile);
our $debugOpened = 0;


sub debug {
	return if (empty($ENV{APBUILD_DEBUG}));

	if (!$debugOpened) {
		if (open DEBUG, '>/dev/tty') {
			$debugOpened = 1;
		} else {
			return;
		}
	}

	my @args = split /\n/, "@_";
	foreach (@args) {
		$_ = '# ' . $_;
		$_ .= "\n";
	}

	print DEBUG "\033[1;33m";
	print DEBUG join '', @args;
	print DEBUG "\033[0m";
	DEBUG->flush;
}


sub empty {
	return !defined($_[0]) || $_[0] eq '';
}

sub homeDir {
	if (!$ENV{HOME}) {
		my $user = getpwuid(POSIX::getuid());
		$ENV{HOME} = (getpwnam($user))[7];
	}
	return $ENV{HOME};
}

# Get the absolute path of a shared library.
# Example: searchLib("libfoo.so.1") --> /usr/lib/libfoo.so.1
sub searchLib {
	my ($basename, @extra_paths) = @_;

	foreach my $path (@extra_paths, '/lib', '/usr/lib', '/usr/local/lib') {
		return "$path/$basename" if (-f "$path/$basename");
	}
	return undef;
}

sub searchStaticLib {
	my ($basename, @extra_paths) = @_;

	foreach my $path (@extra_paths, '/lib', '/usr/lib', '/usr/local/lib') {
		return "$path/$basename" if (-f "$path/$basename");
	}
	return undef;
}

sub soname {
	my ($lib) = @_;
	my ($r, $w);

	if (open2($r, $w, 'objdump', '-p', $lib)) {
		close $w;
		my @lines = <$r>;
		close $r;
		
		my ($soname) = grep {/SONAME/} @lines;
		$soname =~ s/.*?SONAME[ \t]+//;
		$soname =~ s/\n//gs;
		return $soname;

	} else {
		my ($soname) = $lib =~ /.*\/lib(.+)\.so/;
		return $soname;
	}
}

sub parseDataFile {
	my ($file, $r_hash) = @_;

	%{$r_hash} = ();
	return if (!open FILE, "< $file");
	foreach (<FILE>) {
		next if (/^#/);
		s/[\r\n]//g;
		next if (length($_) == 0);

		my ($key, $value) = split /	/, $_, 2;
		$r_hash->{$key} = $value;
	}
	close FILE;
}

sub writeDataFile {
	my ($file, $r_hash) = @_;
	return if (!open FILE, "> $file");
	foreach my $key (sort(keys %{$r_hash})) {
		print FILE "$key	$r_hash->{$key}\n";
	}
	close FILE;
}

1;
